;;; This file defines the following functions which are useful in editing
;;; geomview.tex:
;;;   texify-example:
;;; 	Inserts an '@' in front of each '{', '}', '_', and '$' between
;;; 	"@example" and "@end example".   Point should be somewhere
;;; 	in the example.  Useful for dealing with bits of example code.
;;;   detexify-example
;;; 	Undoes what texify-example does.
;;;   do-include:
;;; 	Replaces the part of the current buffer delimited by the next
;;; 	occurence of lines of the form
;;; 	  @comment #include "FILE"
;;; 	  @comment #end include
;;; 	with the contents of FILE.  This is useful for processing
;;; 	the included example files (example1.c, etc).

(defun change-example (action)
  "Either TeX-ify or de-TeX-ify an example delimited by @example @end example.
   ACTION tells which to do, \"tex\" means TeX-ify it.  Anything else means
   de-TeX-ify it."
  (save-excursion
    (let (beg end)
      (re-search-backward "^@example")
      (next-line 1)
      (setq beg (point))
      (re-search-forward "^@end example")
      (beginning-of-line)
      (setq end (point))
      (narrow-to-region beg end)
      (if (equal action "tex")
	  (progn
	    (goto-char beg)
	    (replace-regexp "\\([{}]\\)" "@\\1")
	    (goto-char beg)
	    (replace-regexp "\\$" "@$")
	    (goto-char beg)
	    (replace-regexp "_" "@_")
	    )
	(progn
	  (goto-char beg)
	  (replace-regexp "@\\([{}]\\)" "\\1")
	  (goto-char beg)
	  (replace-regexp "@\\$" "$")
	  (goto-char beg)
	  (replace-regexp "@_" "_")
	  )
	)
      (widen)
      )
    )
  )

(defun texify-example ()
  "TeXify an example"
  (interactive)
  (change-example "tex")
)

(defun detexify-example ()
  "De-TeX-ify an example"
  (interactive)
  (change-example "detex")
)

(defun do-include ()
  "Replaces the part of the current buffer delimited by the next occurence
of lines of the form
    ``@comment #include \"FILE\"''
    ``@comment #end include''
with the contents of FILE."
  (interactive)
  (let (beg end fname)

;;; get the file name
    (search-forward "@comment #include")
    (search-forward "\"")
    (setq beg (point))
    (search-forward "\"")
    (forward-char -1)
    (setq end (point))
    (setq fname (buffer-substring beg end))

;;; erase old contents of file
    (forward-line 1)
    (beginning-of-line)
    (setq beg (point))
    (search-forward "@comment #end include")
    (beginning-of-line)
    (setq end (point))
    (delete-region beg end)

;;; insert new contents of file
    (insert-file fname)
))
