/* Copyright (c) 1992 The Geometry Center; University of Minnesota
   1300 South Second Street;  Minneapolis, MN  55454, USA;
   
This file is part of geomview/OOGL. geomview/OOGL is free software;
you can redistribute it and/or modify it only under the terms given in
the file COPYING, which you should have received along with this file.
This and other related software may be obtained via anonymous ftp from
geom.umn.edu; email: software@geom.umn.edu. */

/* Authors: Charlie Gunn, Stuart Levy, Tamara Munzner, Mark Phillips */

#include "mgopengl.h"

#ifdef GLUT
# include <GL/glut.h>
#else
# include <GL/gl.h>
# include <GL/glu.h>
# include <GL/glx.h>
#endif


enum { SGL=0, DBL=1 };

typedef struct mgopenglcontext {
  struct mgcontext mgctx;	/* The mgcontext */
  int born;			/* Has window been displayed on the screen? */
  int win;			/* GL window ID, or 0 */
  double zmin, zmax;		/* max z-buffer value */
  short is_PI;			/* Personal Iris? (pgon vs tmesh speed) */
  short turbo;			/* Turbo PI flag: for bug workaround */
  void (*d4f)();		/* For shaded colors: mgopengl_d4f() or glColor4fv() */
  ColorA Cd;			/* Cached diffuse color = mat->diffuse * Kd */
  int lmcolor;			/* lmcolor: GL_DIFFUSE or XXX lmc_COLOR */
  void (*n3f)();		/* For everted normals: glNormal3fv(n) */ 
  Point3 cpos;			/* current-coordinate position of camera
				 * (used for software normal flipping) */

  double znudge;		/* _mgc.zfnudge in integer Z-buffer units */
  double znear, zfar;		/* Current Z-buffer glDepthRange(, $2) limits */
  int oldopts;			/* For knowing when we need to reconfigure */
  vvec room;			/* Scratch space */
#ifndef GLUT
  void *GLXdisplay;		/* X11 Display pointer; if non-NULL, mixed model */
  GLXContext cam_ctx[2];	/* single[0] and double-buf[1] OGL contexts */
  GLXContext curctx;		/* Current OpenGL context */
#endif
  int winids[2];		/* single[0] and double-buffered[1] X win ids */
  int should_lighting, is_lighting; /* Lighting flags so we can turn off
					lighting for points&lines */
  int tevbound;                 /* Texturing currently enabled? */
  TxUser *curtex;              /* Currently-bound texture (NULL if none) */
  int dither;			/* Dither enabled? */
  TxUser *bgimage;		/* Background image (not really a texture) */
} mgopenglcontext;


#define	MAXZNUDGE	8	/* Max possible depth of mgopengl_closer()/farther() calls */

/* We save the current W2C/C2W xforms on each call to mgopengl_worldbegin()
 * because the ModelView matrix stack actually stores both our object
 * xform and the view matrix:
 *
 *   ModelView = [ obj xform ] * [ W2C ]
 *
 * Since we use the GL ModelView stack to keep track of our object xform,
 * instead of using the actual stack in the mgcontext struct, having
 * ModelView = W2C corresponds to [ obj xform ] = identity.  We save W2C
 * and C2W on each mgopengl_worldbegin() because _mgc->cam might change
 * during the course of the frame, but isn't supposed to be reinterpreted
 * until the next mgopengl_worldbegin().
 */

#define _mgopenglc		((mgopenglcontext*)_mgc)

extern void mgopengl_d4f( float c[4] );
extern void mgopengl_n3fevert( Point3 *n, HPoint3 *p );

extern void mgopengl_notexture(void);
extern void mgopengl_needtexture(void);
extern void mgopengl_closer(void);
extern void mgopengl_farther(void);
extern void mgopengl_setshader(mgshadefunc shader);
extern void mgopengl_drawnormal(HPoint3 *p, Point3 *n);
extern void mgopengl_init_zrange();
extern void mgopengl_v4fcloser(HPoint3 *);
extern void mgopengl_txpurge(TxUser *);

#ifdef _WIN32
extern void mgopengl_c4f(float *);
extern void mgopengl_n3f(float *);
#else
#define mgopengl_c4f glColor4fv
#define mgopengl_n3f glNormal3fv
#endif

#define	MAY_LIGHT()  { \
	if(_mgopenglc->should_lighting && !_mgopenglc->is_lighting) { \
	    glEnable(GL_LIGHTING); \
	    _mgopenglc->is_lighting = 1; \
	} }

#define DONT_LIGHT() { \
	    if(_mgopenglc->is_lighting) { \
		glDisable(GL_LIGHTING); \
		_mgopenglc->is_lighting = 0; \
	    } \
	}
